// Header File for Program P.CPP

// P.h
// Header file for P.cpp
// Copyright (c) 1994 Oleg Yarosh

#include <graphics.h>           // Graphics functions

#include <stdio.h>              // FILE,EOF,fgetc(),sprintf()
#include <dos.h>                // sound(), clock()
#include <stdlib.h>             // random(), randomize()
#include <time.h>               // randomize(), CLK_TCK
#include <ctype.h>              // toupper()
#include <conio.h>              // getch()
#include <math.h>               // sin()
#include <iostream.h>           // cout

//------------------------------------------------------------
//  Global data
//------------------------------------------------------------

class par                       // Global parameters
{
public:                         // Public members
  static const int MaxKey;      // Stop after this number of keys
  static const int Delay;       // More delay - slower playing
};

const int par::MaxKey = 20;     // Keys to be pressed
const int par::Delay  = 10;     // Time delay (millisecomds)

const int MaxAmp = 20;          // Max trace x amplitude
const int MinAmp = 10;          // Min trace x amplitude

const int Esc   = 27;           // Key to exit (escape)

const int XStep = 20;           // Grid step
const int YStep = 20;

//------------------------------------------------------------
//  Enumerations
//------------------------------------------------------------

enum  {
        SkyColor   = LIGHTCYAN,
        CharColor  = BLUE,
        TraceColor = LIGHTCYAN,
        SnowColor  = WHITE,
        TextColor  = YELLOW,
        ValueColor = YELLOW,
        GroundColor= GREEN,
        BarColor   = BLUE
};

enum  beep  {BeepAtt,BeepErr};

//------------------------------------------------------------
//  Class key
//------------------------------------------------------------

class   key                   // Class declaration
{
private:                      // Private members
  FILE *pFILE;                // Pointer to FILE structure
  enum hand {Left,Right};     // Local enum type
  hand Hand;                  // Hand for key

  char *Set;                  // Character set to display
  int Len;                    // Its length

  int Init();                 // Initialize graphics system
  void WhatHand();            // Key for what hand?

public:                       // Public members
  key();                      // Constructor
  ~key();                     // Destructor
  char Char;                  // Displaing character
  int Input();                // Input key set from file
  void Get();                 // Get next random char
  void Draw();
  void Clear();               // Clear keyboard
  int Trace (int & Key);      // Display trace, get key
};

//------------------------------------------------------------
//  Class key object definition
//
//  Call constructor to initialize graphics system before main()
//  Call destructor to close graphics system after main()
//------------------------------------------------------------

key K;

//------------------------------------------------------------
//  Class point
//------------------------------------------------------------

struct point 
{
  int X;
  int Y;
  point () { X=0; Y=0; }
  point (float x,float y);
};

//------------------------------------------------------------
//  Points on the screen (must be defined after K object)
//------------------------------------------------------------

point LeftPeak    (8,    6),  // Mountain peak
      RightPeak   (12,   4),
      LeftChar    (8,    5),  // Character
      RightChar   (12,   3),
      LeftStart   (8,    7),  // Trace start
      RightStart  (12,   7),
      LeftFinish  (8,    16), // Trace finish
      RightFinish (12,   16),
      TopBar      (9.8,  7),  // Moving bar
      BottomBar   (10.2, 16),
      ScoreText   (3,    18),
      ErrorText   (14,   18);

//------------------------------------------------------------
//  Functions
//------------------------------------------------------------

void Play ();
int  Pause ();
void Sound (beep Tone);
void DrawPicture ();
void DisplayScore (int Score);
void DisplayErr (int ErrNum);
void DisplayBar (int Count);
void GameOver ();