//  Program 16.1
//  Playing with C++. Version 1 (text mode)
	
#include <iostream.h>             // cout
#include <ctype.h>                // toupper()
#include <conio.h>                // getch(),kbhit()
#include <stdlib.h>               // random(),randomize()
#include <dos.h>                  // sound(),nosound(),delay()
#include <stdio.h>                // FILE,EOF,fopen(),fgetc(),fclose()
	
//    Global data

const int MaxKey  = 20;           // Stop after MaxKey times
const int Delay   = 30;           // More delay - slower playing
const int MaxChar = 40;           // Number of running characters
	
const int Esc     = 27;           // Escape key code - for exit

//    Types
	
enum  beep {BeepAtt,BeepErr};

//    Function declarations	

int InputKeySet(char * KeySet, int & NKeys);
int Run(char Char, int & Key);
void Sound(beep Tone);
	
int main()
{
  int Score   = 0;                // Game score
  int Key     = 0;                // Last key pressed
  int KeyNum  = 0;                // Number of keys pressed
	
  char Char;                      // Running character
  char KeySet[1000];              // Key set, used for playing
  int  NKeys;                     // Actual number of used keys
	
  NKeys = sizeof KeySet;
  if (InputKeySet(KeySet,NKeys))  // Input key set from file,
                                  //   set actual NKeys
    return 1;                     // Return on file open error
	
  cout << "Press any key to start...\n";
  getch ();                       // Wait for key pressed
  randomize ();                   // For a random start key

  while (KeyNum++ < MaxKey)       // Main loop <= MaxKey times
  {
    Char=KeySet[random(NKeys)];   // Get random character
    Sound (BeepAtt);              // Attention beep
    Score += Run (Char,Key=0);    // Run character, get Key
    if (Key == Esc)               // Exit if Esc was pressed
      break;
    if (Key != Char)
      Sound (BeepErr);            // Error beep
  }

//    Display score
	
  cout << "\nScore " << (Score > 0 ? Score : 0);
  return 0;
}
	
int  InputKeySet(char * KeySet, int & NKeys) 
{
  /*    Input key set
        from the first line of the file P16-01.dat
        to the array of characters KeySet
        Reset NKeys to the actual number of imputed keys
	
        Return
          0 - on success
          1 - on error
  */
	
  FILE *  pFILE;                  // Pointer to FILE structure

  pFILE=fopen ("P16-01.DAT","rt");// Open data file
  if (! pFILE)
  {
    cout << "Unable to open file!!!";
    return 1;                     // Return 1 on open error
  }
	
  int i = 0;
  while (i < NKeys)               // Loop <= sizeof(KeySet) times
  {
    int Key = fgetc (pFILE);      // Get character from a file
    Key = toupper (Key);          // Uppercase it
    if (Key == EOF)  
      break;                      // Exit on end of file
    if (Key == '\n') 
      break;                      // Exit on end of LINE  <--
    if (Key == ' ') 
      continue;                   // Ignore blanks
    KeySet[i++] = (char) Key;     // Initialize KeySet
  }
  NKeys = i;                      // Number of inputted keys
	
  fclose (pFILE);
  return 0;                 // Close data file
}
	

int  Run(char Char, int & Key)
{
  for (int i = 0; i < MaxChar; i++)  // Running character loop
  {
    cout << Char;                    // Output a character
    delay(Delay);                    // Delay to slow speed
    if (kbhit() )                    // Test for key pressed
    {
      Key = toupper (getch ());      // Get key from the keyboard
      if (! Key)
	getch();                     // If not ASCII character
      break;                         // Exit loop
    }
  }
  cout << endl;                      // Go to new line

  int Score;

  if (i == MaxChar)
    Score = -MaxChar;
  else 
    if (Char != Key) 
      Score = -MaxChar/4;
    else
      Score =  MaxChar - i;

  return Score;
}
	
void Sound(beep Tone) 
{
  int Freq=0;                        // Frequency, Hz
  int Delay=0;                       // Time (milliseconds)
	
  const int AttFreq  = 110;          // Hz
  const int ErrFreq  = 440;          // Hz
  const int AttDelay = 50;           // msec
  const int ErrDelay = 200;          // msec
	
  switch (Tone) 
  {
    case BeepAtt: 
      Freq=AttFreq; 
      Delay=AttDelay; 
      break;
    case BeepErr: 
      Freq=ErrFreq; 
      Delay=ErrDelay; 
      break;
  }
  sound (Freq);
  delay (Delay);
  nosound ();
  delay (Delay);
}